<?php

/**
 * @file
 * Drupal vB system settings.
 *
 * Most parts forked from Migrator module, http://drupal.org/project/migrator
 */

/**
 * Form builder function for DrupalvB integration settings.
 */
function drupalvb_settings_integration() {

  $form['drupalvb_path'] = array(
    '#type' => 'textfield',
    '#title' => t('vBulletin installation path'),
    '#default_value' => variable_get('drupalvb_path', ''),
    '#size' => 40,
    '#description' => t('Enter the path to your vBulletin installation. This must be a relative or absolute file system path. Example: <code>./forum</code><br />If your vBulletin installation is on a different server and not locally accessible, copy the contents of the <code>forum/includes</code> directory somewhere on this server and fill in the path to that directory.'),
  );
  $form['drupalvb_license'] = array(
    '#type' => 'textfield',
    '#title' => t('vBulletin license number'),
    '#default_value' => variable_get('drupalvb_license', ''),
    '#size' => 12,
    '#description' => t('Enter your vBulletin license number, which can be found at the top of any vBulletin PHP file. This is required to generate proper session id hashes for cookies. Note that this is not your vBulletin customer number.'),
  );
  $form['drupalvb_dual_login'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable concurrent user login'),
    '#default_value' => variable_get('drupalvb_dual_login', TRUE),
    '#description' => t('When enabled, users are also logged in to vBulletin when they log into Drupal. For accounts that do not exist in vBulletin yet, this will only work if the account creation option below is also enabled.'),
  );
  // @todo Rename to 'drupalvb_account_create'.
  $form['drupalvb_acct_generation'] = array(
    '#type' => 'checkbox',
    '#title' => t('Create vBulletin accounts'),
    '#default_value' => variable_get('drupalvb_acct_generation', TRUE),
    '#description' => t('When enabled, vBulletin accounts are created for Drupal users upon registration and login (the latter only if enabled above).'),
  );
  // @todo Rename to 'drupalvb_account_sync'.
  $form['drupalvb_acct_sync'] = array(
    '#type' => 'checkbox',
    '#title' => t('Synchronize account updates'),
    '#default_value' => variable_get('drupalvb_acct_sync', TRUE),
    '#description' => t('When enabled, the username, password, e-mail address, roles, language, and timezone settings are synchronized with vBulletin when a Drupal account is updated.'),
  );

  return system_settings_form($form);
}

/**
 * Form builder function for DrupalvB roles settings.
 *
 * @todo Migrate drupalvb_default_usergroup to drupalvb_usergroup for all Drupal
 *   roles. 13/05/2009 smk
 */
function drupalvb_settings_roles() {
  $form = array();
  $roles = user_roles(TRUE);
  $usergroups = drupalvb_get_usergroups();

  // Roles will be reused multiple times below.
  foreach ($roles as $rid => $role) {
    $form['role'][$rid] = array('#value' => $role);
  }

  // @todo Use tabledrag in D6 and merge with primary usergroup table.
  $form['role_weights_help'] = array(
    '#type' => 'item',
    '#title' => t('Role weights'),
    '#description' => t('Select the weight of each Drupal role. This will determine the primary usergroup a user gets assigned to in the table below. Roles with a lower weight will be considered first.'),
  );
  $form['drupalvb_role_weights'] = array('#tree' => TRUE);
  $role_weights = variable_get('drupalvb_role_weights', array());
  foreach ($roles as $rid => $role) {
    $form['drupalvb_role_weights'][$rid] = array(
      '#type' => 'weight',
      '#default_value' => isset($role_weights[$rid]) ? $role_weights[$rid] : 0,
    );
  }

  $form['usergroup_help'] = array(
    '#type' => 'item',
    '#title' => t('Primary usergroup'),
    '#description' => t('Select the primary usergroup for each Drupal role. Note that if a user has more than one role, the role with the <strong>lowest weight set above</strong> will determine which usergroup is applied.'),
  );
  $form['drupalvb_usergroup'] = array('#tree' => TRUE);
  $options = array();
  foreach ($usergroups as $gid => $title) {
    $options[$gid] = $title;
  }
  $usergroup = variable_get('drupalvb_usergroup', array());
  foreach ($roles as $rid => $role) {
    $form['drupalvb_usergroup'][$rid] = array(
      '#type' => 'select',
      '#default_value' => isset($usergroup[$rid]) ? $usergroup[$rid] : 2,
      '#options' => $options,
    );
  }

  $form['membergroups_help'] = array(
    '#type' => 'item',
    '#title' => t('Additional usergroups'),
    '#description' => t('Select any additional usergroup(s) users should be assigned to. This is optional.'),
  );

  $form['checkboxes']['drupalvb_membergroups'] = array('#tree' => TRUE);
  $options = array();
  foreach ($usergroups as $gid => $title) {
    $options[$gid] = '';
    $form['usergroups'][$gid] = array('#value' => $title, '#tree' => TRUE);
  }
  $membergroups = variable_get('drupalvb_membergroups', array());
  foreach ($roles as $rid => $role) {
    $form['checkboxes']['drupalvb_membergroups'][$rid] = array('#type' => 'checkboxes', '#options' => $options, '#default_value' => $membergroups[$rid]);
  }

  $form['#submit'] = array('drupalvb_settings_roles_submit' => array());

  return system_settings_form($form);
}

/**
 * Form submit handler for user role settings form.
 */
function drupalvb_settings_roles_submit($form_id, &$form_values) {
  variable_set('drupalvb_role_weights', array_map('intval', array_filter($form_values['drupalvb_role_weights'])));
  variable_set('drupalvb_usergroup', array_map('intval', $form_values['drupalvb_usergroup']));
  foreach ($form_values['drupalvb_membergroups'] as $rid => $usergroups) {
    $form_values['drupalvb_membergroups'][$rid] = array_map('intval', array_filter($usergroups));
  }
  variable_set('drupalvb_membergroups', array_filter($form_values['drupalvb_membergroups']));
}

/**
 * Theme role matrix like user access permissions form.
 */
function theme_drupalvb_settings_roles($form) {
  // Role weights table.
  $rows = array();
  $output = drupal_render($form['role_weights_help']);
  foreach (element_children($form['role']) as $rid) {
    // Skip internal Form API values.
    if (is_array($form['role'][$rid])) {
      $cells = array();
      $cells[] = array('data' => drupal_render($form['role'][$rid]), 'class' => 'role', 'width' => '50%');
      $cells[] = array('data' => drupal_render($form['drupalvb_role_weights'][$rid]), 'align' => 'center');
    }
    $rows[] = $cells;
  }
  $header = array(t('Role'), t('Weight'));
  $output .= theme('table', $header, $rows, array('id' => 'role-weights', 'width' => '50%'));

  // Primary usergroup table.
  $rows = array();
  $output .= drupal_render($form['usergroup_help']);
  foreach (element_children($form['role']) as $rid) {
    if (is_array($form['role'][$rid])) {
      $cells = array();
      // Reset element status to re-use already rendered form element.
      // @todo Simply use $form['role'][$rid]['#children'] instead. 29/05/2009 sun
      unset($form['role'][$rid]['#printed']);
      $cells[] = array('data' => drupal_render($form['role'][$rid]), 'class' => 'role', 'width' => '50%');
      $cells[] = array('data' => drupal_render($form['drupalvb_usergroup'][$rid]), 'align' => 'center');
    }
    $rows[] = $cells;
  }
  $header = array(t('Role'), t('Primary usergroup'));
  $output .= theme('table', $header, $rows, array('id' => 'usergroup', 'width' => '50%'));

  // Membergroups matrix table.
  $rows = array();
  $output .= drupal_render($form['membergroups_help']);
  foreach (element_children($form['role']) as $rid) {
    if (is_array($form['role'][$rid])) {
      $cells = array();
      // @todo Simply use $form['role'][$rid]['#children'] instead. 29/05/2009 sun
      unset($form['role'][$rid]['#printed']);
      $cells[] = array('data' => drupal_render($form['role'][$rid]), 'class' => 'role');
      foreach (element_children($form['usergroups']) as $ugid) {
        if (is_array($form['checkboxes']['drupalvb_membergroups'][$ugid])) {
          $cells[] = array('data' => drupal_render($form['checkboxes']['drupalvb_membergroups'][$rid][$ugid]));
          $form['checkboxes']['drupalvb_membergroups'][$rid]['#printed'] = TRUE;
        }
      }
      $rows[] = $cells;
    }
  }
  $header = array(array('data' => t('Role'), 'style' => 'width: 20%; text-align: left; vertical-align: bottom;'));
  foreach (element_children($form['usergroups']) as $ugid) {
    if (is_array($form['usergroups'][$ugid])) {
      $header[] = array('data' => drupal_render($form['usergroups'][$ugid]), 'style' => 'width: 10%; text-align: left; vertical-align: bottom; -moz-transform: rotate(-60deg); -moz-transform-origin: 0% 75%;');
    }
  }
  $output .= theme('table', $header, $rows, array('id' => 'membergroups'));
  $output .= drupal_render($form);
  return $output;
}

/**
 * Form builder for remote system database connection settings.
 */
function drupalvb_settings_database() {
  $form = array();
  $db = parse_url(variable_get('drupalvb_db', is_array($GLOBALS['db_url']) ? $GLOBALS['db_url']['default'] : $GLOBALS['db_url']));
  $form['db'] = array(
    '#type' => 'fieldset',
    '#title' => t('Database connection'),
  );
  $form['db']['scheme'] = array(
    '#type' => 'radios',
    '#title' => t('Database interface'),
    '#options' => array('mysql' => 'MySQL', 'mysqli' => 'MySQLi'),
    '#default_value' => $GLOBALS['db_type'], // D7: $db['scheme'],
    '#disabled' => TRUE,
    '#description' => t('Due to the database abstraction design in Drupal 5 and 6, only the current default database interface is supported.'),
  );
  $form['db']['host'] = array(
    '#type' => 'textfield',
    '#title' => t('Host'),
    '#default_value' => !empty($db['host']) ? $db['host'] : 'localhost',
    '#required' => TRUE,
  );
  $form['db']['port'] = array(
    '#type' => 'textfield',
    '#title' => t('Port'),
    '#default_value' => !empty($db['port']) ? $db['port'] : '3306',
    '#required' => TRUE,
  );
  $form['db']['path'] = array(
    '#type' => 'textfield',
    '#title' => t('Database'),
    '#default_value' => substr($db['path'], 1),
    '#required' => TRUE,
  );
  $form['db']['user'] = array(
    '#type' => 'textfield',
    '#title' => t('Username'),
    '#default_value' => $db['user'],
    '#required' => TRUE,
  );
  $form['db']['pass'] = array(
    '#type' => 'textfield',
    '#title' => t('Password'),
    '#default_value' => $db['pass'],
    '#required' => TRUE,
  );
  $form['db']['db_prefix'] = array(
    '#type' => 'textfield',
    '#title' => t('Table prefix'),
    '#default_value' => variable_get('drupalvb_db_prefix', 'vb_'),
  );
  $form[] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  return $form;
}

/**
 * Form submit handler for remote system database connection settings.
 */
function drupalvb_settings_database_submit($form_id, $form_values) {
  // Upon initial database configuration we setup the initial user mapping.
  // @todo This breaks if wrong database settings have been submitted, since
  // the initial mapping will never be created. Use result of drupalvb_valid_db()
  // instead.
  $initial_import = !variable_get('drupalvb_db', 0) ? TRUE : FALSE;

  $url = $form_values['scheme'] .'://'. $form_values['user'] .':'. $form_values['pass'] .'@'. $form_values['host'] .':'. $form_values['port'] .'/'. $form_values['path'];
  variable_set('drupalvb_db', $url);
  variable_set('drupalvb_db_is_default', (is_array($GLOBALS['db_url']) ? $GLOBALS['db_url']['default'] == $url : $GLOBALS['db_url'] == $url));
  variable_set('drupalvb_db_prefix', $form_values['db_prefix']);

  if ($initial_import) {
    // First time setup; initialize user mapping.
    require_once drupal_get_path('module', 'drupalvb') .'/drupalvb.inc';
    _drupalvb_init_user_map();
  }
}

/**
 * Form builder for export actions.
 */
function drupalvb_settings_export() {
  $form = array();
  $form[] = array(
    '#type' => 'item',
    '#title' => t('Select an action to perform'),
  );
  $form['action']['action'] = array(
    '#type' => 'value',
  );
  $form['action']['forum'] = array(
    '#type' => 'radio',
    '#name' => 'action',
    '#return_value' => 'forum',
    '#title' => t('Export Drupal forum to vBulletin'),
    '#disabled' => !module_exists('forum'),
    '#description' => t('Exports the complete Drupal forum to vBulletin.  Unlike the user export below, it will copy only users which have posted in the past.  To make all users available in vBulletin simply run the user export before or afterwards.'),
  );
  $form['action']['users'] = array(
    '#type' => 'radio',
    '#name' => 'action',
    '#return_value' => 'users',
    '#title' => t('Export Drupal users to vBulletin'),
    '#description' => t('Exports the complete Drupal user base to vBulletin.<br /><strong>Warning:</strong> This export relies on usernames.  If identical usernames exist in both systems, the user vBulletin account will be overwritten with data from Drupal.'),
  );
  $form['action']['privatemsg'] = array(
    '#type' => 'radio',
    '#name' => 'action',
    '#return_value' => 'privatemsg',
    '#title' => t('Export Drupal privatemsg counts to vBulletin'),
    '#disabled' => !module_exists('privatemsg'),
    '#description' => t('Updates vBulletin user private message statistics with counts from privatemsg module.'),
  );
  if (!module_exists('batch')) {
    $form['action']['#description'] = t('<a href="@batchapi-url">Batch API</a> module is required for exporting data.', array('@batchapi' => url('http://drupal.org/project/batchapi')));
    foreach (element_children($form['action']) as $element) {
      $form['action'][$element]['#disabled'] = TRUE;
    }
  }

  $form[] = array(
    '#type' => 'item',
    '#title' => t('Advanced options'),
  );
  $form['clean'] = array(
    '#type' => 'checkbox',
    '#title' => t('Clean mapping tables before export'),
    '#description' => t('Empty internal Drupal to vBulletin mapping tables which this module uses to incrementally export data.  Check this if you have reset your vBulletin installation and want to start from scratch.'),
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Start'),
  );
  return $form;
}

/**
 * Form submit handler for export actions form.
 */
function drupalvb_settings_export_submit($form_id, $form_values) {
  if (!$form_values['action']) {
    drupal_set_message(t('Please select an export option.'), 'error');
    return;
  }
  return 'admin/settings/drupalvb/export/confirm/' . $form_values['action'] . '/' . $form_values['clean'];
}

/**
 * Form builder function for export confirm form.
 */
function drupalvb_export_confirm_form($action, $clean) {
  if (!$action) {
    return drupal_not_found();
  }

  $form['action'] = array('#type' => 'hidden', '#value' => $action);
  $form['clean'] = array('#type' => 'hidden', '#value' => $clean);

  $question = t('Are you sure you want to export?');

  if ($clean) {
    $description = t("<strong>Warning:</strong> You have selected the 'clean' option.  This will reset existing mappings and should <strong>only ever</strong> be required if you are starting over with a fresh (empty) vBulletin installation.  Otherwise it will create duplicates.");
    $description .= ' ' . t('This action cannot be undone.');
    $button = t('Clean + Export');
  }
  else {
    $description = t('This action cannot be undone.');
    $button = t('Export');
  }

  return confirm_form($form, $question, 'admin/settings/drupalvb/export', $description, $button);
}

function drupalvb_export_confirm_form_submit($form_id, $form_values) {
  $path = drupal_get_path('module', 'drupalvb');
  require_once $path .'/drupalvb.inc';

  switch ($form_values['action']) {
    case 'forum':
      // Reset mappings (cannot be done during batch).
      if ($form_values['clean']) {
        drupalvb_reset_mapping(array('users', 'forums', 'threads', 'posts'));
      }

      // Initialize batch.
      $batch = array(
        'title' => t('Exporting forum'),
        'operations' => array(),
        'file' => $path . '/drupalvb.batch.inc',
      );
      batch_set($batch);

      // Export forum categories (single operation).
      $batch = array(
        'title' => t('Exporting forum'),
        'operations' => array(array('drupalvb_export_forum_categories', array())),
        'finished' => 'drupalvb_export_finished',
        'file' => $path . '/drupalvb.batch.inc',
      );
      batch_set($batch);

      // Export forum threads (progressive).
      $batch = array(
        'title' => t('Exporting forum'),
        'operations' => array(),
        'finished' => 'drupalvb_export_finished',
        'file' => $path . '/drupalvb.batch.inc',
      );
      $result = db_query("SELECT nid FROM {node} WHERE status = 1 AND type = 'forum' ORDER BY nid ASC");
      while ($node = db_fetch_array($result)) {
        $batch['operations'][] = array('drupalvb_export_forum_thread', array($node['nid']));
      }
      // Add the rebuild operation to the end.
      $batch['operations'][] = array('drupalvb_export_forum_rebuild', array());
      batch_set($batch);
      break;

    case 'users':
      // Reset mappings (cannot be done during batch).
      if ($form_values['clean']) {
        drupalvb_reset_mapping(array('users'));
      }

      // Initialize batch.
      $batch = array(
        'title' => t('Exporting users'),
        'operations' => array(),
        'finished' => 'drupalvb_export_finished',
        'file' => $path . '/drupalvb.batch.inc',
      );
      batch_set($batch);

      // Export users (progressive).
      $batch = array(
        'title' => t('Exporting users'),
        'operations' => array(),
        'file' => $path . '/drupalvb.batch.inc',
      );
      $result = db_query("SELECT u.uid
        FROM {users} u
        LEFT JOIN {drupalvb_map} m ON m.type = 'users' AND u.uid = m.drupal_id
        WHERE u.uid > 0 AND m.vb_id IS NULL
        ORDER BY u.uid ASC");
      while ($user = db_fetch_array($result)) {
        $batch['operations'][] = array('drupalvb_export_user', array($user['uid']));
      }
      batch_set($batch);
      break;

    case 'privatemsg':
      // Initialize batch.
      $batch = array(
        'title' => t('Exporting privatemsg counts'),
        'operations' => array(),
        'finished' => 'drupalvb_export_finished',
        'file' => $path . '/drupalvb.batch.inc',
      );
      batch_set($batch);

      // Reset pm counters.
      drupalvb_db_query("UPDATE {user} SET pmtotal = 0, pmunread = 0");

      // Export privatemsg counts (progressive).
      $batch = array(
        'title' => t('Exporting privatemsg counts'),
        'operations' => array(),
        'file' => $path . '/drupalvb.batch.inc',
      );
      $result = db_query("SELECT recipient, COUNT(*) AS pmtotal, SUM(IF(newmsg = 1, 1, 0)) AS pmunread FROM {privatemsg} WHERE recipient_del = 0 GROUP BY recipient");
      while ($user = db_fetch_array($result)) {
        $batch['operations'][] = array('drupalvb_export_privatemsg', array($user['recipient'], $user['pmtotal'], $user['pmunread']));
      }
      batch_set($batch);
      break;
  }

  batch_process('admin/settings/drupalvb/export');
}

/**
 * Form builder for vBulletin variables (Devel).
 */
function drupalvb_settings_variables() {
  $form = array();
  $options = drupalvb_get('options');
  foreach ($options as $key => $value) {
    $form['variables'][$key]['name'] = array('#value' => check_plain($key));
    $form['variables'][$key]['value'] = array('#value' => check_plain($value));
  }
  ksort($form['variables']);
  return $form;
}

/**
 * Theme vBulletin variables like Drupal variables (Devel).
 */
function theme_drupalvb_settings_variables(&$form) {
  $header = array(t('Name'), t('Value'));
  $rows = array();
  foreach (element_children($form['variables']) as $key) {
    $rows[] = array(
      drupal_render($form['variables'][$key]['name']),
      drupal_render($form['variables'][$key]['value']),
    );
  }
  $output = theme('table', $header, $rows);
  $output .= drupal_render($form);
  return $output;
}

