<?php

/**
 * @file
 * DrupalvB batch export operation callbacks.
 */

/**
 * Export Drupal forum categories to vBulletin forums.
 */
function drupalvb_export_forum_categories(&$context, $parent = 0) {
  // @todo Move into global scope of this file; required for all operations.
  require_once drupal_get_path('module', 'drupalvb') .'/drupalvb.inc';

  // Need to re-index display order because vB chokes on negative values.
  $displayorder = 1;

  // Root forums in Drupal should be moved into a category in vBulletin,
  // otherwise the administration page gets cluttered.
  $forumid = 1;
  $main_category = (fetch_foruminfo($forumid) ? $forumid : -1);

  // Fetch one level of the term hierarchy.
  $forums = taxonomy_get_tree(variable_get('forum_nav_vocabulary', ''), $parent, -1, 1);
  foreach ($forums as $forum) {
    if (!drupalvb_get_vb_id('forums', $forum->tid)) {
      $forum->container = in_array($forum->tid, variable_get('forum_containers', array()));
    
      // Create forum.
      $vbforum = &drupalvb_datamanager_init('Forum');
      $vbforum->set_info('disable_cache_rebuild', TRUE);

      $vbforum->set('title', $forum->name);
      $vbforum->set('description', $forum->description);
      $vbforum->set('displayorder', $displayorder);
      $vbforum->set('daysprune', -1);
      $parentid = array_shift($forum->parents);
      if ($parentid > 0) {
        // Regular forums are placed at their parent.
        $parentid = drupalvb_get_vb_id('forums', $parentid);
      }
      elseif ($forum->container) {
        // Containers are placed at the root level.
        $parentid = -1;
      }
      else {
        // Root forums are moved into a category.
        $parentid = $main_category;
      }
      $vbforum->set('parentid', $parentid);
  
      // Geez, vB has absolutely no defaults for forum options...
      $vbforum->set_bitfield('options', 'cancontainthreads', !$forum->container);
      $vbforum->set_bitfield('options', 'active', TRUE);
      $vbforum->set_bitfield('options', 'allowposting', !$forum->container);
      $vbforum->set_bitfield('options', 'indexposts', TRUE);
  
      $vbforum->set_bitfield('options', 'allowbbcode', TRUE);
      $vbforum->set_bitfield('options', 'allowimages', TRUE);
      $vbforum->set_bitfield('options', 'allowsmilies', TRUE);
      $vbforum->set_bitfield('options', 'allowicons', TRUE);
      $vbforum->set_bitfield('options', 'allowratings', TRUE);
      $vbforum->set_bitfield('options', 'countposts', TRUE);
      $vbforum->set_bitfield('options', 'showonforumjump', TRUE);
  
      $vbforum->pre_save();
      if ($vbforum->has_errors()) {
        $context['results']['errors'][] = t('Failed creating category for !tid: !errors', array('!tid' => l('forum/'. $forum->tid, 'forum/'. $forum->tid), '!errors' => theme('item_list', $vbforum->errors)));
      }
      else {
        $vbforum->save();
        $forumid = $vbforum->fetch_field('forumid');
        drupalvb_set_mapping('forums', $forum->tid, $forumid);
      }
      unset($vbforum);
    }

    // Recurse into child forums.
    drupalvb_export_forum_categories($context, $forum->tid);

    $displayorder++;
  }
}

/**
 * Export a Drupal forum node to a vBulletin thread (with first post).
 */
function drupalvb_export_forum_thread($nid, &$context) {
  require_once drupal_get_path('module', 'drupalvb') .'/drupalvb.inc';

  // Create thread with first post.
  $vbthread = &drupalvb_datamanager_init('Thread_FirstPost', 'threadpost');
  $vbthread->set_info('is_automated', TRUE);

  if (!$node = node_load($nid)) {
    return;
  }
  $forumid = drupalvb_get_vb_id('forums', $node->tid);
  if ($threadid = drupalvb_get_vb_id('threads', $node->nid)) {
    $existing = fetch_threadinfo($threadid);
    $vbthread->set_existing($existing);
  }
  else {
    if (!$forumid) {
      $context['results']['errors'][] = t('Missing forum mapping for !tid', array('!tid' => l('forum/'. $node->tid, 'forum/'. $node->tid)));
      return;
    }
    $vbthread->set('forumid', $forumid);
    $vbthread->set('open', 1);
    $vbthread->set('prefixid', '');
    $vbthread->set('visible', 1);
    $vbthread->set('allowsmilie', 1);
    $vbthread->set('showsignature', 1);
    // Do not assign the IP address of the exporting user.
    $vbthread->set('ipaddress', '');
  }

  $vbthread->set('title', $node->title);
  $userid = drupalvb_get_vb_id('users', $node->uid);
  if (!$userid) {
    // Create account first.
    $account = user_load(array('uid' => $node->uid));
    if (!$account->uid) {
      return;
    }
    $userid = drupalvb_synchronize_user($account);
    if (!$userid) {
      $context['results']['errors'][] = t('Failed creating user @name', array('@name' => $account->name));
      return;
    }
  }
  $vbthread->set('userid', $userid);
  // Since vBulletin orders posts by date, we can't use last change date here.
  $vbthread->set('dateline', $node->created);
  $vbthread->set('pagetext', drupalvb_html2bb($node->body));

  $vbthread->pre_save();
  if ($vbthread->has_errors()) {
    $context['results']['errors'][] = t('Failed creating thread from node !nid: !errors', array('!nid' => l('node/'. $node->nid, 'node/'. $node->nid), '!errors' => theme('item_list', $vbthread->errors)));
  }
  else {
    $vbthread->save();
    $threadid = $vbthread->fetch_field('threadid');
    drupalvb_set_mapping('threads', $node->nid, $threadid);

    // Create follow-up posts.
    drupalvb_export_forum_posts($node->nid, $threadid, $context);

    // Save ids for rebuilding counts later.
    $context['results']['forumids'][$forumid] = TRUE;
    $context['results']['threadids'][$threadid] = TRUE;
    $context['results']['userids'][$userid] = TRUE;
  }

  $context['message'] = t('Exported thread @title', array('@title' => $node->title));
}

/**
 * Export Drupal forum node comments to vBulletin posts.
 */
function drupalvb_export_forum_posts($nid, $threadid, &$context) {
  // Get node comments.
  $query = "SELECT cid, uid, name, subject, comment, format, hostname, timestamp FROM {comments} WHERE status = %d AND nid = %d";
  $mode = variable_get('comment_default_mode', COMMENT_MODE_THREADED_EXPANDED);
  if ($mode == COMMENT_MODE_FLAT_COLLAPSED || $mode == COMMENT_MODE_FLAT_EXPANDED) {
    $query .= ' ORDER BY cid';
  }
  else {
    $query .= ' ORDER BY SUBSTRING(thread, 1, (LENGTH(thread) - 1))';
  }
  $result = db_query($query, COMMENT_PUBLISHED, $nid);

  while ($comment = db_fetch_object($result)) {
    $vbpost = &drupalvb_datamanager_init('Post', 'threadpost');
    $vbpost->set_info('is_automated', TRUE);

    if ($postid = drupalvb_get_vb_id('posts', $comment->cid)) {
      $existing = fetch_postinfo($postid);
      $vbpost->set_existing($existing);
    }
    else {
      $vbpost->set('threadid', $threadid);
      $vbpost->set('visible', 1);
      $vbpost->set('allowsmilie', 1);
      $vbpost->set('showsignature', 1);
    }

    if ($comment->uid) {
      $userid = drupalvb_get_vb_id('users', $comment->uid);
      if (!$userid) {
        // Create account first.
        $account = user_load(array('uid' => $comment->uid));
        if (!$account->uid) {
          unset($vbpost);
          continue;
        }
        $userid = drupalvb_synchronize_user($account);
        if (!$userid) {
          $context['results']['errors'][] = t('Failed mapping user @name', array('@name' => $account->name));
          unset($vbpost);
          continue;
        }
      }
      $vbpost->set('userid', $userid);
      $context['results']['userids'][$userid] = TRUE;
    }
    else {
      // @todo Test anonymous posts. Also required for threads.
      $vbpost->set('username', $comment->name);
    }
    // Get rid of Drupal's impertinence.
    // @todo This constantly crashes the Apache webserver.
//    $default_subject = trim(truncate_utf8(decode_entities(strip_tags(check_markup($comment->comment, $comment->format, FALSE))), 29, TRUE));
//    if ($comment->subject != $default_subject && $comment->subject != t('(No subject)')) {
      $vbpost->set('title', $comment->subject);
//    }
//    else {
//      $vbpost->set('title', '');
//    }
    $vbpost->set('dateline', $comment->timestamp);
    $vbpost->set('pagetext', drupalvb_html2bb($comment->comment));
    $vbpost->set('ipaddress', $comment->hostname);

    $vbpost->pre_save();
    if ($vbpost->has_errors()) {
      $context['results']['errors'][] = t('Failed creating post from comment !comment: !errors', array('!comment' => l('node/'. $nid .'/'. $comment->cid, 'node/'. $nid .'/'. $comment->cid), '!errors' => theme('item_list', $vbpost->errors)));
    }
    else {
      $vbpost->save();
      $postid = $vbpost->fetch_field('postid');
      drupalvb_set_mapping('posts', $comment->cid, $postid);
    }
    unset($vbpost);
  }
}

/**
 * Rebuild vBulletin counters after threads export has finished.
 */
function drupalvb_export_forum_rebuild(&$context) {
  $path = drupal_get_path('module', 'drupalvb');
  require_once $path .'/drupalvb.inc';
  drupalvb_bootstrap();
  global $vbulletin;

  // Rebuild users.
  if (!empty($context['results']['userids'])) {
    $batch = array(
      'title' => t('Updating counters'),
      'operations' => array(),
      'file' => $path . '/drupalvb.batch.inc',
    );
    $forums = array();
    $result = drupalvb_db_query("SELECT forumid FROM {forum} WHERE (options & " . $vbulletin->bf_misc_forumoptions['countposts'] . ")");
    while ($forum = db_fetch_array($result)) {
      $forums[] = $forum['forumid'];
    }
    $forums = implode(',', $forums);

    foreach (array_keys($context['results']['userids']) as $userid) {
      $batch['operations'][] = array('drupalvb_export_forum_rebuild_user', array($userid, $forums));
    }
    batch_set($batch);
    unset($context['results']['userids']);
  }

  // Rebuild threads.
  if (!empty($context['results']['threadids'])) {
    $batch = array(
      'title' => t('Updating counters'),
      'operations' => array(),
      'file' => $path . '/drupalvb.batch.inc',
    );
    foreach (array_keys($context['results']['threadids']) as $threadid) {
      $batch['operations'][] = array('drupalvb_export_forum_rebuild_thread', array($threadid));
    }
    batch_set($batch);
    unset($context['results']['threadids']);
  }

  // Rebuild forums.
  if (!empty($context['results']['forumids'])) {
    $batch = array(
      'title' => t('Updating counters'),
      'operations' => array(),
      'file' => $path . '/drupalvb.batch.inc',
    );
    foreach (array_keys($context['results']['forumids']) as $forumid) {
      $batch['operations'][] = array('drupalvb_export_forum_rebuild_forum', array($forumid));
    }
    // Rebuild forum permissions last.
    $batch['operations'][] = array('drupalvb_export_forum_rebuild_forum', array(NULL));
    batch_set($batch);
    unset($context['results']['forumids']);
  }
}

/**
 * Rebuild user post counters.
 */
function drupalvb_export_forum_rebuild_user($userid, $forums, &$context) {
  require_once drupal_get_path('module', 'drupalvb') .'/drupalvb.inc';

  $vbuser = &drupalvb_datamanager_init('User');
  $existing = array('userid' => $userid);
  $vbuser->set_existing($existing);
  $stats = db_fetch_array(drupalvb_db_query("SELECT MAX(p.postid) AS lastpostid, MAX(p.dateline) AS lastpost, COUNT(*) AS posts
    FROM {post} p
    INNER JOIN {thread} t ON t.threadid = p.threadid
    WHERE p.userid = %d AND t.forumid IN (%s) AND t.visible = 1 AND p.visible = 1", $userid, $forums));
  $vbuser->set('posts', $stats['posts']);
  $vbuser->set_ladder_usertitle($stats['posts']);
  $vbuser->set('lastpost', $stats['lastpost']);
  $vbuser->set('lastpostid', $stats['lastpostid']);
  $vbuser->save();

  $context['message'] = t('Updating user information');
}

/**
 * Rebuild thread counters.
 */
function drupalvb_export_forum_rebuild_thread($threadid, &$context) {
  require_once drupal_get_path('module', 'drupalvb') .'/drupalvb.inc';
  drupalvb_bootstrap();

  require_once DIR . '/includes/functions_databuild.php';
  build_thread_counters($threadid);

  $context['message'] = t('Updating thread information');
}

/**
 * Rebuild forum counters.
 */
function drupalvb_export_forum_rebuild_forum($forumid, &$context) {
  require_once drupal_get_path('module', 'drupalvb') .'/drupalvb.inc';
  drupalvb_bootstrap();

  if (!empty($forumid)) {
    require_once DIR . '/includes/functions_databuild.php';
    build_forum_counters($forumid, TRUE);
  }
  else {
    // Rebuild forum permissions.
    require_once(DIR . '/includes/adminfunctions.php');
    unset($GLOBALS['vbulletin']->forumcache);
    build_forum_permissions();
  }

  $context['message'] = t('Updating forum information');
}

/**
 * Export a Drupal user to vBulletin.
 */
function drupalvb_export_user($uid, &$context) {
  require_once drupal_get_path('module', 'drupalvb') .'/drupalvb.inc';

  $account = user_load(array('uid' => $uid));
  if ($account->uid) {
    drupalvb_synchronize_user($account);
    $context['message'] = t('Exported user @username', array('@username' => $account->name));
  }
}

/**
 * Update privatemsg counts in vBulletin.
 */
function drupalvb_export_privatemsg($recipient, $pmtotal, $pmunread, &$context) {
  require_once drupal_get_path('module', 'drupalvb') .'/drupalvb.inc';

  // Verify that recipient exists in vB.
  $userid = drupalvb_get_vb_id('users', $recipient);
  if (!$userid) {
    $account = user_load(array('uid' => $recipient));
    if (!$userid = drupalvb_synchronize_user($account)) {
      return;
    }
  }

  // Update counts for user.
  $vbuser = &drupalvb_datamanager_init('User');
  $existing = array('userid' => $userid);
  $vbuser->set_existing($existing);
  $vbuser->set('pmtotal', $pmtotal);
  $vbuser->set('pmunread', $pmunread);
  $vbuser->save();
}

function drupalvb_export_finished($success, $results, $operations) {
  if (!empty($results['errors'])) {
    foreach ($results['errors'] as $error) {
      drupal_set_message($error, 'error');
    }
    $message = t('Finished with errors.');
  }
  else {
    $message = t('Finished successfully.');
  }
  drupal_set_message($message);
}

/**
 * Convert HTML into BBCode.
 *
 * Idea borrowed from D7's new HTML Corrector filter.
 */
function drupalvb_html2bb($text) {
  // DOM can load HTML soup. But, HTML soup can throw warnings, suppress them.
  @$html = DOMDocument::loadHTML('<html><head><meta http-equiv="Content-Type" content="text/html; charset=utf-8" /></head><body>' . $text . '</body></html>');
  // vBulletin chokes on UTF-8 encoded &nbsp; in database, replace with regular
  // space.
  return str_replace("\xC2\xA0", ' ', _drupalvb_html2bb($html));
}

function _drupalvb_html2bb($element) {
  $content = '';
  foreach ($element->childNodes as $node) {
    if ($node->nodeType == XML_TEXT_NODE) {
      $content .= $node->nodeValue;
    }
    else {
      // @todo Kill this switch by setting up a mapping array.
      // @todo Add support for more BBCode tag attributes.
      switch ($node->nodeName) {
        case 'a':
          $href = trim($node->getAttribute('href'));
          if ($node->hasChildNodes()) {
            $inner = trim(_drupalvb_html2bb($node));
          }
          if (isset($inner) && $inner != $href) {
            $content .= '[url=' . $href . ']';
            $content .= $inner;
            $content .= '[/url]';
          }
          else {
            $content .= '[url]';
            $content .= $href;
            $content .= '[/url]';
          }
          break;

        case 'strong':
          if ($node->hasChildNodes()) {
            $content .= '[b]';
            $content .= trim(_drupalvb_html2bb($node));
            $content .= '[/b]';
          }
          break;

        case 'em':
          if ($node->hasChildNodes()) {
            $content .= '[i]';
            $content .= trim(_drupalvb_html2bb($node));
            $content .= '[/i]';
          }
          break;

        case 'u':
          if ($node->hasChildNodes()) {
            $content .= '[u]';
            $content .= trim(_drupalvb_html2bb($node));
            $content .= '[/u]';
          }
          break;

        case 'blockquote':
          if ($node->hasChildNodes()) {
            $content .= '[quote]';
            $content .= trim(_drupalvb_html2bb($node));
            $content .= '[/quote]';
            $content .= "\n";
          }
          break;

        case 'ol':
          if ($node->hasChildNodes()) {
            $content .= "[list=1]\n";
            $content .= trim(_drupalvb_html2bb($node));
            $content .= '[/list]';
            $content .= "\n";
          }
          break;

        case 'ul':
          if ($node->hasChildNodes()) {
            $content .= "[list]\n";
            $content .= trim(_drupalvb_html2bb($node));
            $content .= '[/list]';
            $content .= "\n";
          }
          break;

        case 'li':
          if ($node->hasChildNodes()) {
            $content .= '[*] ';
            $content .= trim(_drupalvb_html2bb($node));
            $content .= "\n";
          }
          break;

        case 'p':
          if ($node->hasChildNodes()) {
            $content .= _drupalvb_html2bb($node);
          }
          $content .= "\n";
          break;

        case 'br':
          if ($node->hasChildNodes()) {
            $content .= _drupalvb_html2bb($node);
          }
          break;

        default:
          if ($node->hasChildNodes()) {
            $content .= _drupalvb_html2bb($node);
          }
          break;
      }
    }
  }
  return $content;
}

