<?php

/**
 * @file
 * Drupal vB CRUD functions.
 */

/**
 * Set the necessary cookies for the user to be logged into the forum.
 *
 * Frontend cookie names:
 * - lastvisit, lastactivity, sessionhash
 * Backend cookie names:
 * - cpsession, userid, password
 *
 * However, in all cases the cookiedomain is NOT prefixed with a dot unless
 * cookie domain has not been manually altered to either a suggested value or
 * custom value in vB's settings.
 */
function drupalvb_set_login_cookies($userid) {
  // Load required vB user data.
  $vbuser = db_fetch_array(drupalvb_db_query("SELECT userid, password FROM {user} WHERE userid = %d", $userid));
  if (!$vbuser) {
    return FALSE;
  }
  
  $vb_config = drupalvb_get('config');
  $vb_options = drupalvb_get('options');

  $cookie_prefix = (isset($vb_config['Misc']['cookieprefix']) ? $vb_config['Misc']['cookieprefix'] : 'bb');
  $cookie_path = $vb_options['cookiepath'];
  $now = time();
  $expire = $now + (@ini_get('session.cookie_lifetime') ? ini_get('session.cookie_lifetime') : 60 * 60 * 24 * 365);

  $vb_cookie_domain = (!empty($vb_options['cookiedomain']) ? $vb_options['cookiedomain'] : $GLOBALS['cookie_domain']);
  // Per RFC 2109, cookie domains must contain at least one dot other than the
  // first. For hosts such as 'localhost' or IP Addresses we don't set a cookie domain.
  // @see conf_init()
  if (!(count(explode('.', $vb_cookie_domain)) > 2 && !is_numeric(str_replace('.', '', $vb_cookie_domain)))) {
    $vb_cookie_domain = '';
  }

  // Clear out old session (if available).
  if (!empty($_COOKIE[$cookie_prefix .'sessionhash'])) {
    drupalvb_db_query("DELETE FROM {session} WHERE sessionhash = '%s'", $_COOKIE[$cookie_prefix .'sessionhash']);
  }

  // Setup user session.
  $ip = implode('.', array_slice(explode('.', drupalvb_get_ip()), 0, 4 - $vb_options['ipcheck']));
  $idhash = md5($_SERVER['HTTP_USER_AGENT'] . $ip);
  $sessionhash = md5($now . request_uri() . $idhash . $_SERVER['REMOTE_ADDR'] . user_password(6));

  drupalvb_db_query("REPLACE INTO {session} (sessionhash, userid, host, idhash, lastactivity, location, useragent, loggedin) VALUES ('%s', %d, '%s', '%s', %d, '%s', '%s', %d)", $sessionhash, $vbuser['userid'], substr($_SERVER['REMOTE_ADDR'], 0, 15), $idhash, $now, '/forum/', $_SERVER['HTTP_USER_AGENT'], 2);

  // Setup cookies.
  setcookie($cookie_prefix .'sessionhash', $sessionhash, $expire, $cookie_path, $vb_cookie_domain);
  setcookie($cookie_prefix .'lastvisit', $now, $expire, $cookie_path, $vb_cookie_domain);
  setcookie($cookie_prefix .'lastactivity', $now, $expire, $cookie_path, $vb_cookie_domain);
  setcookie($cookie_prefix .'userid', $vbuser['userid'], $expire, $cookie_path, $vb_cookie_domain);
  setcookie($cookie_prefix .'password', md5($vbuser['password'] . variable_get('drupalvb_license', '')), $expire, $cookie_path, $vb_cookie_domain);
  return TRUE;
}

/**
 * Clear all vB cookies for the current user.
 *
 * @see drupalvb_logout(), drupalvb_user_logout()
 */
function drupalvb_clear_cookies($userid = NULL) {
  $vb_config = drupalvb_get('config');
  $vb_options = drupalvb_get('options');

  $cookie_prefix = (isset($vb_config['Misc']['cookieprefix']) ? $vb_config['Misc']['cookieprefix'] : 'bb');
  $cookie_path = $vb_options['cookiepath'];
  $expire = time() - 86400;

  $vb_cookie_domain = (!empty($vb_options['cookiedomain']) ? $vb_options['cookiedomain'] : $GLOBALS['cookie_domain']);
  // Per RFC 2109, cookie domains must contain at least one dot other than the
  // first. For hosts such as 'localhost' or IP Addresses we don't set a cookie domain.
  // @see conf_init()
  if (!(count(explode('.', $vb_cookie_domain)) > 2 && !is_numeric(str_replace('.', '', $vb_cookie_domain)))) {
    $vb_cookie_domain = '';
  }

  // @todo Without a vB user id, we cannot delete the session, so vBulletin
  //   will automatically authenticate again. We badly need a solution here,
  //   since this is the cause for broken session handling. Proposal: Use
  //   drupalvb_get_ip() to count the # of sessions; if there exactly one,
  //   kill it.
  if (!empty($userid)) {
    drupalvb_db_query("DELETE FROM {session} WHERE userid = %d", $userid);
    drupalvb_db_query("UPDATE {user} SET lastvisit = %d WHERE userid = %d", time(), $userid);
  }

  setcookie($cookie_prefix .'sessionhash', '', $expire, $cookie_path, $vb_cookie_domain);
  setcookie($cookie_prefix .'lastvisit', '', $expire, $cookie_path, $vb_cookie_domain);
  setcookie($cookie_prefix .'lastactivity', '', $expire, $cookie_path, $vb_cookie_domain);
  setcookie($cookie_prefix .'userid', '', $expire, $cookie_path, $vb_cookie_domain);
  setcookie($cookie_prefix .'password', '', $expire, $cookie_path, $vb_cookie_domain);
}

/**
 * Determines the IP address of current user.
 *
 * @todo Duplicate of ip_address() in D6+ ?
 */
function drupalvb_get_ip() {
  $ip = $_SERVER['REMOTE_ADDR'];

  if (isset($_SERVER['HTTP_CLIENT_IP'])) {
    $ip = $_SERVER['HTTP_CLIENT_IP'];
  }
  else if (isset($_SERVER['HTTP_X_FORWARDED_FOR']) && preg_match_all('#\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}#s', $_SERVER['HTTP_X_FORWARDED_FOR'], $matches)) {
    // Try to avoid using an internal IP address, it's probably a proxy.
    $ranges = array(
      '10.0.0.0/8' => array(ip2long('10.0.0.0'), ip2long('10.255.255.255')),
      '127.0.0.0/8' => array(ip2long('127.0.0.0'), ip2long('127.255.255.255')),
      '169.254.0.0/16' => array(ip2long('169.254.0.0'), ip2long('169.254.255.255')),
      '172.16.0.0/12' => array(ip2long('172.16.0.0'), ip2long('172.31.255.255')),
      '192.168.0.0/16' => array(ip2long('192.168.0.0'), ip2long('192.168.255.255')),
    );
    foreach ($matches[0] as $_ip) {
      $ip_long = ip2long($_ip);
      if ($ip_long === FALSE || $ip_long == -1) {
        continue;
      }

      $private_ip = FALSE;
      foreach ($ranges as $range) {
        if ($ip_long >= $range[0] && $ip_long <= $range[1]) {
          $private_ip = TRUE;
          break;
        }
      }

      if (!$private_ip) {
        $ip = $_ip;
        break;
      }
    }
  }
  else if (isset($_SERVER['HTTP_FROM'])) {
    $ip = $_SERVER['HTTP_FROM'];
  }

  return $ip;
}

/**
 * Synchronize a Drupal user account with vBulletin.
 *
 * @param $account
 *   A Drupal user account object.
 * @param $edit
 *   Submitted form values, usually provided by hook_user().
 * @return
 *   The matching vBulletin userid for the account. FALSE indicates a
 *   unrecoverable vBulletin data validation error.
 */
function drupalvb_synchronize_user($account, $edit = array()) {
  define('DRUPALVB_DISABLE_SYNC', TRUE);

  // Query the remote database whether the given username is known.
  if ($userid = db_result(drupalvb_db_query("SELECT userid FROM {user} WHERE LOWER(username) = LOWER('%s') LIMIT 1", drupalvb_htmlspecialchars($account->name)))) {
    // Update existing vBulletin account with data in $edit.
    if (!_drupalvb_update_user($userid, $account, $edit)) {
      return FALSE;
    }
  }
  else {
    // Create vBulletin user from existing account data.
    if (!$userid = _drupalvb_create_user((array) $account)) {
      return FALSE;
    }
  }
  // Ensure that a user mapping exists.
  drupalvb_set_mapping('users', $account->uid, $userid);
  return $userid;
}

/**
 * Create a user in vBulletin.
 *
 * @see drupalvb_synchronize_user()
 */
function _drupalvb_create_user($edit) {
  $vbuser = &drupalvb_datamanager_init('User');
  global $vbulletin, $db;

  // Disable several validations that do not exist in Drupal.
  $vbuser->adminoverride = TRUE;

  $vbuser->set('username', $edit['name']);
  $vbuser->set('email', $edit['mail']);
  $vbuser->set('password', $edit['pass']);
  $vbuser->set('joindate', $edit['created']);
  $vbuser->set('lastvisit', $edit['access']);
  $vbuser->set('lastactivity', $edit['access']);

  // Determine target usergroups.
  $vbuser->set('usergroupid', drupalvb_get_usergroup($edit['roles']));
  $vbuser->set('membergroupids', drupalvb_get_membergroups($edit['roles']));

  // Determine timezone from user preference (requires date_timezone.module),
  // or from system setting.
  $timezone = isset($edit['timezone']) ? $edit['timezone'] : variable_get('date_default_timezone', 0);
  // vBulletin stores hours, not seconds.
  $vbuser->set('timezoneoffset', $timezone / 3600);

  $vbuser->pre_save();

  if ($vbuser->has_errors()) {
    foreach ($vbuser->errors as $error) {
      watchdog('drupalvb', t('Failed creating forum user %username: !error', array('%username' => $edit['name'], '!error' => $error)), WATCHDOG_ERROR);
    }
    return FALSE;
  }

  $userid = $vbuser->save();

  // Notify crappy plugins that fail to use datamanager hooks.
  ($hook = vBulletinHook::fetch_hook('register_addmember_complete')) ? eval($hook) : FALSE;

  watchdog('drupalvb', t('Created forum user %username', array('%username' => $edit['name'])));

  return $userid;
}

/**
 * Update a user in vBulletin.
 *
 * @see drupalvb_synchronize_user()
 */
function _drupalvb_update_user($userid, $account, $edit) {
  $vbuser = &drupalvb_datamanager_init('User');
  // Disable several validations that do not exist in Drupal.
  $vbuser->adminoverride = TRUE;

  $existing = array('userid' => $userid);
  $vbuser->set_existing($existing);
  // Always update last activity timestamps.
  $vbuser->set('lastvisit', $account->access);
  $vbuser->set('lastactivity', $account->access);

  foreach ($edit as $field => $value) {
    switch ($field) {
      case 'name':
        $vbuser->set('username', $value);
        break;

      case 'roles':
        $vbuser->set('usergroupid', drupalvb_get_usergroup($value));
        $vbuser->set('membergroupids', drupalvb_get_membergroups($value));
        break;

      case 'pass':
        // vBulletin's datamanager accepts both a plain or a single-hashed
        // password (but not hashed + salted).
        $vbuser->set('password', $value);
        break;

      case 'mail':
        $vbuser->set('email', $value);
        break;

      case 'language':
        $vbuser->set('languageid', drupalvb_get('languageid', $value));
        break;

      case 'timezone':
        // vBulletin stores hours, not seconds.
        $vbuser->set('timezoneoffset', $value / 3600);
        break;

      case 'guestbook_visited':
        // Update visitor message notification on behalf of guestbook.module.
        // This will only be set for the owner.
        $vbuser->set('vmunreadcount', 0);
        break;
    }
  }

  $vbuser->pre_save();

  if ($vbuser->has_errors()) {
    foreach ($vbuser->errors as $error) {
      watchdog('drupalvb', t('Failed updating forum user %username: !error', array('%username' => $account->name, '!error' => $error)), WATCHDOG_ERROR);
    }
    return FALSE;
  }

  $vbuser->save();
  watchdog('drupalvb', t('Updated forum user %username', array('%username' => $account->name)));

  return TRUE;
}

/**
 * Retrieve primary role (lowest configured role weight) from one or more roles.
 *
 * @param $roles
 *   An array of Drupal roles.
 * @return
 *   Primary Drupal role.
 */
function drupalvb_get_primary_role($roles) {
  $primary_role = DRUPAL_AUTHENTICATED_RID;
  $primary_weight = 0;
  if (!empty($roles)) {
    $role_weights = variable_get('drupalvb_role_weights', array());
    foreach (array_keys($roles) as $rid) {
      if (isset($role_weights[$rid]) && $role_weights[$rid] < $primary_weight) {
        $primary_role = $rid;
        $primary_weight = $role_weights[$rid];
      }
    }
  }
  return $primary_role;
}

/**
 * Retrieve primary usergroup from one or more roles.
 *
 * @param $roles
 *   An array of Drupal roles.
 * @return
 *   A vBulletin usergroup id.
 */
function drupalvb_get_usergroup($roles) {
  $rid = drupalvb_get_primary_role($roles);
  $usergroup = variable_get('drupalvb_usergroup', array());
  return isset($usergroup[$rid]) ? $usergroup[$rid] : 2;
}

/**
 * Retrieve additional usergroups (membergroups) for one or more roles.
 *
 * @param $roles
 *   An array of Drupal roles.
 * @return
 *   All matching vBulletin membergroup ids as comma separated string.
 */
function drupalvb_get_membergroups($roles) {
  $rid = drupalvb_get_primary_role($roles);
  $membergroups = variable_get('drupalvb_membergroups', array());
  return isset($membergroups[$rid]) ? implode(',', $membergroups[$rid]) : '';
}

/**
 * Lookup a Drupal id for a given vBulletin id.
 *
 * @param $type
 *   Entity type. Currently supported are 'users', 'forums', 'threads', and
 *   'posts'.
 * @param $vb_id
 *   A vBulletin id.
 */
function drupalvb_get_drupal_id($type, $vb_id) {
  return db_result(db_query("SELECT drupal_id FROM {drupalvb_map} WHERE type = '%s' AND vb_id = %d", $type, $vb_id));
}

/**
 * Lookup a vBulletin id for a given Drupal id.
 *
 * @param $type
 *   Entity type. Currently supported are 'users', 'forums', 'threads', and
 *   'posts'.
 * @param $drupal_id
 *   A Drupal id.
 */
function drupalvb_get_vb_id($type, $drupal_id) {
  return db_result(db_query("SELECT vb_id FROM {drupalvb_map} WHERE type = '%s' AND drupal_id = %d", $type, $drupal_id));
}

/**
 * Store mapping between two entities.
 *
 * @param $type
 *   Entity type. Currently supported are 'users', 'forums', 'threads', and
 *   'posts'.
 * @param $drupal_id
 *   A Drupal id.
 * @param $vb_id
 *   A vBulletin id.
 */
function drupalvb_set_mapping($type, $drupal_id, $vb_id) {
  db_query("INSERT IGNORE INTO {drupalvb_map} (type, drupal_id, vb_id) VALUES ('%s', %d, %d)", $type, $drupal_id, $vb_id);
}

/**
 * Flush all stored mappings.
 *
 * @param $types
 *   Either a single or an array of entity types. Currently supported are
 *   'users', 'forums', 'threads', and 'posts'.
 */
function drupalvb_reset_mapping($types) {
  if (is_string($types)) {
    $types = array($types);
  }
  $placeholders = implode(',', array_fill(0, count($types), "'%s'"));
  db_query("DELETE FROM {drupalvb_map} WHERE type IN (". $placeholders .")", $types);
  if (in_array('users', $types)) {
    // Re-initialize user map.
    _drupalvb_init_user_map();
  }
}

/**
 * Get vBulletin configuration options.
 */
function drupalvb_get_options() {
  static $options;

  if (!isset($options)) {
    $result = db_query("SELECT varname, value FROM {setting}");
    while ($var = db_fetch_array($result)) {
      $options[$var['varname']] = $var['value'];
    }
  }
  return (array) $options;
}

/**
 * Get vBulletin configuration.
 */
function drupalvb_get_config() {
  static $config;

  // @todo Find & include vB's config automatically?
  // $files = file_scan_directory('.', '^config.php$', $nomask = array('.', '..', 'CVS', '.svn'));
  $config_file = drupal_get_path('module', 'drupalvb') .'/config.php';
  if (!isset($config) && file_exists($config_file)) {
    require_once $config_file;
  }
  return (array) $config;
}

/**
 * Get vBulletin usergroups.
 */
function drupalvb_get_usergroups() {
  $usergroups = array();
  $result = drupalvb_db_query("SELECT usergroupid, title FROM {usergroup} WHERE usergroupid > 1 ORDER BY title ASC");
  while ($data = db_fetch_object($result)) {
    $usergroups[$data->usergroupid] = $data->title;
  }
  return $usergroups;
}

/**
 * Get vB language id by given ISO language code.
 */
function drupalvb_get_languageid($language = NULL) {
  static $vblanguages;

  if (!isset($vblanguages)) {
    $vblanguages = array();
    $result = drupalvb_db_query("SELECT languageid, title, languagecode FROM {language}");
    while ($lang = db_fetch_array($result)) {
      $vblanguages[$lang['languagecode']] = $lang['languageid'];
    }
  }
  $options = drupalvb_get('options');
  return (!empty($language) && isset($vblanguages[$language]) ? $vblanguages[$language] : $vblanguages[$options['languageid']]);
}

/**
 * Get counts of guests and members currently online.
 */
function drupalvb_get_users_online() {
  $vb_options = drupalvb_get('options');

  $datecut          = time() - $vb_options['cookietimeout'];
  $numbervisible    = 0;
  $numberregistered = 0;
  $numberguest      = 0;

  $result = drupalvb_db_query("SELECT user.username, user.usergroupid, session.userid, session.lastactivity FROM {session} AS session LEFT JOIN {user} AS user ON (user.userid = session.userid) WHERE session.lastactivity > %d", $datecut);

  $userinfos = array();

  while ($loggedin = db_fetch_array($result)) {
    $userid = $loggedin['userid'];
    if (!$userid) {
      $numberguest++;
    }
    else if (empty($userinfos[$userid]) || ($userinfos[$userid]['lastactivity'] < $loggedin['lastactivity'])) {
      $userinfos[$userid] = $loggedin;
    }
  }
  foreach ($userinfos as $userid => $loggedin) {
    $numberregistered++;
  }
  return array('guests' => $numberguest, 'members' => $numberregistered);
}

/**
 * Get counts of new or recent posts for the current user.
 */
function drupalvb_get_recent_posts($scope = 'last') {
  global $user;

  // Queries the vB user database to find a matching set of user data.
  $result = drupalvb_db_query("SELECT userid, username, lastvisit FROM {user} WHERE username = '%s'", drupalvb_htmlspecialchars($user->name));

  // Make sure a user is logged in to get their last visit and appropriate post
  // count.
  if ($vb_user = db_fetch_array($result)) {
    if ($scope == 'last') {
      $datecut = $vb_user['lastvisit'];
    }
    else if ($scope == 'daily') {
      $datecut = time() - 86400;
    }
    $posts = db_result(drupalvb_db_query("SELECT COUNT(postid) FROM {post} WHERE dateline > %d", $datecut));
  }
  else {
    $posts = 0;
  }
  return $posts;
}

function drupalvb_htmlspecialchars($text) {
  $text = preg_replace('/&(?!#[0-9]+|shy;)/si', '&amp;', $text);
  return str_replace(array('<', '>', '"'), array('&lt;', '&gt;', '&quot;'), $text);
}

